/* jshint latedef:false */
/* jshint forin:false */
/* jshint noempty:false */

// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

'use strict';

var util = require('util');
var _ = require('underscore');
var azureCommon = require('azure-common');
var xml = azureCommon.xml2js;
var js2xml = azureCommon.js2xml;
var Service = azureCommon.Service;
var WebResource = azureCommon.WebResource;

var SqlManagementClient = ( /** @lends SqlManagementClient */ function() {
  /**
   * @class
   * The SQL Database Management API is a REST API for managing SQL Database
   * servers and the firewall rules associated with SQL Database servers.
   * (see http://msdn.microsoft.com/en-us/library/windowsazure/gg715283.aspx
   * for more information)
   * Initializes a new instance of the SqlManagementClient class.
   * @constructor
   * 
   * @param {SubscriptionCloudCredentials} credentials When you create a
   * Windows Azure subscription, it is uniquely identified by a subscription
   * ID. The subscription ID forms part of the URI for every call that you
   * make to the Service Management API.  The Windows Azure Service
   * ManagementAPI use mutual authentication of management certificates over
   * SSL to ensure that a request made to the service is secure.  No anonymous
   * requests are allowed.
   * 
   * @param {string} [credentials.subscriptionId]
   * 
   * @param {object} [credentials.managementCertificate]
   * 
   * @param {string} [baseUri] The URI used as the base for all SQL requests.
   * 
   * @param {Array} filters
   */
  function SqlManagementClient(credentials, baseUri, filters) {
    if (credentials === null || credentials === undefined) {
      throw new Error('credentials cannot be null.');
    }
    
    SqlManagementClient['super_'].call(this, credentials, filters);
    
    this.credentials = credentials;
    this.baseUri = baseUri;
    if (this.baseUri === null || this.baseUri === undefined) {
      this.baseUri = 'https://management.core.windows.net';
    }
    /**
     * Provides an instance of the [DacOperations](-DacOperations.html) object.
     * @type {object}
     */
    this.dac = new DacOperations(this);
    /**
     * Provides an instance of the
     * [DatabaseOperationOperations](-DatabaseOperationOperations.html) object.
     * @type {object}
     */
    this.databaseOperations = new DatabaseOperationOperations(this);
    /**
     * Provides an instance of the
     * [DatabaseOperations](-DatabaseOperations.html) object.
     * @type {object}
     */
    this.databases = new DatabaseOperations(this);
    /**
     * Provides an instance of the
     * [FirewallRuleOperations](-FirewallRuleOperations.html) object.
     * @type {object}
     */
    this.firewallRules = new FirewallRuleOperations(this);
    /**
     * Provides an instance of the [ServerOperations](-ServerOperations.html)
     * object.
     * @type {object}
     */
    this.servers = new ServerOperations(this);
    /**
     * Provides an instance of the
     * [ServiceObjectiveOperations](-ServiceObjectiveOperations.html) object.
     * @type {object}
     */
    this.serviceObjectives = new ServiceObjectiveOperations(this);
  }
  
  util.inherits(SqlManagementClient, Service);
  
  
  return SqlManagementClient;
})();
exports.SqlManagementClient = SqlManagementClient;

var DacOperations = ( /** @lends DacOperations */ function() {
  /**
   * @class
   * Includes operations for importing and exporting SQL Databases into and out
   * of Windows Azure blob storage.
   * __NOTE__: An instance of this class is automatically created for an
   * instance of the [SqlManagementClient] {@link
   * SqlManagementClient~SqlManagementClient}.
   * See [dac] {@link SqlManagementClient~SqlManagementClient#dac}.
   * Initializes a new instance of the DacOperations class.
   * @constructor
   * 
   * @param {SqlManagementClient} client Reference to the service client.
   */
  function DacOperations(client) {
    this.client = client;
  }
  
  /**
   * Export DAC into Windows Azure blob storage.
   * 
   * @param {string} serverName The name of the server being exported from.
   * 
   * @param {DacExportParameters} [parametersOrCallback] Export parameters.
   * 
   * @param {BlobCredentialsParameter} [parametersOrCallback.blobCredentials]
   * Gets or sets credentials for getting the DAC.
   * 
   * @param {string} [parametersOrCallback.blobCredentials.uri] Gets or sets
   * the URI of the DAC file stored in Windows Azure Blob Storage.
   * 
   * @param {string} [parametersOrCallback.blobCredentials.storageAccessKey]
   * Gets or sets the key for the Windows Azure Storage account.
   * 
   * @param {ConnectionInfoParameter} [parametersOrCallback.connectionInfo]
   * Gets or sets connection information for the SQL Server Database.
   * 
   * @param {string} [parametersOrCallback.connectionInfo.databaseName] Gets or
   * sets the name of the database.
   * 
   * @param {string} [parametersOrCallback.connectionInfo.password] Gets or
   * sets the password for the database user.
   * 
   * @param {string} [parametersOrCallback.connectionInfo.serverName] Gets or
   * sets the SQL server name.
   * 
   * @param {string} [parametersOrCallback.connectionInfo.userName] Gets or
   * sets the username of the database user.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  DacOperations.prototype.export = function(serverName, parametersOrCallback, callback) {
    var parameters = parametersOrCallback;
    if (_.isFunction(parameters)) {
      callback = parameters;
      parameters = null;
    }
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (parameters) {
      if (parameters.blobCredentials) {
        if (parameters.blobCredentials.storageAccessKey === null || parameters.blobCredentials.storageAccessKey === undefined) {
          return callback(new Error('parameters.blobCredentials.storageAccessKey cannot be null.'));
        }
        if (parameters.blobCredentials.uri === null || parameters.blobCredentials.uri === undefined) {
          return callback(new Error('parameters.blobCredentials.uri cannot be null.'));
        }
      }
      if (parameters.connectionInfo) {
        if (parameters.connectionInfo.databaseName === null || parameters.connectionInfo.databaseName === undefined) {
          return callback(new Error('parameters.connectionInfo.databaseName cannot be null.'));
        }
        if (parameters.connectionInfo.password === null || parameters.connectionInfo.password === undefined) {
          return callback(new Error('parameters.connectionInfo.password cannot be null.'));
        }
        if (parameters.connectionInfo.serverName === null || parameters.connectionInfo.serverName === undefined) {
          return callback(new Error('parameters.connectionInfo.serverName cannot be null.'));
        }
        if (parameters.connectionInfo.userName === null || parameters.connectionInfo.userName === undefined) {
          return callback(new Error('parameters.connectionInfo.userName cannot be null.'));
        }
      }
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/DacOperations/Export';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    if (parameters) {
      var exportInputElement = js2xml.createElement('ExportInput', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
      js2xml.addChildElement(requestDoc, exportInputElement);
      
      if (parameters.blobCredentials) {
        var blobCredentialsElement = js2xml.createElement('BlobCredentials', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.addChildElement(exportInputElement, blobCredentialsElement);
        
        var typeAttribute = js2xml.createAttribute('type', 'http://www.w3.org/2001/XMLSchema-instance');
        js2xml.setAttributeValue(typeAttribute, 'BlobStorageAccessKeyCredentials');
        js2xml.addAttribute(blobCredentialsElement, typeAttribute);
        
        var uriElement = js2xml.createElement('Uri', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(uriElement, parameters.blobCredentials.uri);
        js2xml.addChildElement(blobCredentialsElement, uriElement);
        
        var storageAccessKeyElement = js2xml.createElement('StorageAccessKey', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(storageAccessKeyElement, parameters.blobCredentials.storageAccessKey);
        js2xml.addChildElement(blobCredentialsElement, storageAccessKeyElement);
      }
      
      if (parameters.connectionInfo) {
        var connectionInfoElement = js2xml.createElement('ConnectionInfo', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.addChildElement(exportInputElement, connectionInfoElement);
        
        var databaseNameElement = js2xml.createElement('DatabaseName', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(databaseNameElement, parameters.connectionInfo.databaseName);
        js2xml.addChildElement(connectionInfoElement, databaseNameElement);
        
        var passwordElement = js2xml.createElement('Password', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(passwordElement, parameters.connectionInfo.password);
        js2xml.addChildElement(connectionInfoElement, passwordElement);
        
        var serverNameElement = js2xml.createElement('ServerName', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(serverNameElement, parameters.connectionInfo.serverName);
        js2xml.addChildElement(connectionInfoElement, serverNameElement);
        
        var userNameElement = js2xml.createElement('UserName', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(userNameElement, parameters.connectionInfo.userName);
        js2xml.addChildElement(connectionInfoElement, userNameElement);
      }
    }
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var guidElement = js2xml.getElement(responseDoc, responseDoc, 'GUID', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/');
        if (guidElement) {
          result.guid = guidElement;
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Gets the status of the DAC.
   * 
   * @param {string} serverName The name of the server.
   * 
   * @param {string} fullyQualifiedServerName The fully qualified name of the
   * server.
   * 
   * @param {string} username The server's username.
   * 
   * @param {string} password The server's password.
   * 
   * @param {string} requestId The request ID of the operation being queried.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  DacOperations.prototype.getStatus = function(serverName, fullyQualifiedServerName, username, password, requestId, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (fullyQualifiedServerName === null || fullyQualifiedServerName === undefined) {
      return callback(new Error('fullyQualifiedServerName cannot be null.'));
    }
    if (username === null || username === undefined) {
      return callback(new Error('username cannot be null.'));
    }
    if (password === null || password === undefined) {
      return callback(new Error('password cannot be null.'));
    }
    if (requestId === null || requestId === undefined) {
      return callback(new Error('requestId cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/DacOperations/Status?';
    url2 = url2 + 'servername=' + encodeURIComponent(fullyQualifiedServerName.trim());
    url2 = url2 + '&username=' + encodeURIComponent(username.trim());
    url2 = url2 + '&password=' + encodeURIComponent(password.trim());
    url2 = url2 + '&reqId=' + encodeURIComponent(requestId.trim());
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { statusInfoList: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var arrayOfStatusInfoElement = js2xml.getElement(responseDoc, responseDoc, 'ARRAYOFSTATUSINFO', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
        if (arrayOfStatusInfoElement) {
          if (arrayOfStatusInfoElement) {
            for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, arrayOfStatusInfoElement, 'STATUSINFO', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES').length; loweredIndex1 = loweredIndex1 + 1) {
              var statusInfoElement = js2xml.getElements(responseDoc, arrayOfStatusInfoElement, 'STATUSINFO', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES')[loweredIndex1];
              var statusInfoInstance = {};
              result.statusInfoList.push(statusInfoInstance);
              
              var blobUriElement = js2xml.getElement(responseDoc, statusInfoElement, 'BLOBURI', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (blobUriElement) {
                var blobUriInstance = blobUriElement;
                statusInfoInstance.blobUri = blobUriInstance;
              }
              
              var databaseNameElement = js2xml.getElement(responseDoc, statusInfoElement, 'DATABASENAME', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (databaseNameElement) {
                var databaseNameInstance = databaseNameElement;
                statusInfoInstance.databaseName = databaseNameInstance;
              }
              
              var errorMessageElement = js2xml.getElement(responseDoc, statusInfoElement, 'ERRORMESSAGE', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (errorMessageElement) {
                var isNil = false;
                var nilAttribute = js2xml.getAttribute(responseDoc, errorMessageElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                if (nilAttribute) {
                  isNil = nilAttribute === 'true';
                }
                if (isNil === false) {
                  var errorMessageInstance = errorMessageElement;
                  statusInfoInstance.errorMessage = errorMessageInstance;
                }
              }
              
              var lastModifiedTimeElement = js2xml.getElement(responseDoc, statusInfoElement, 'LASTMODIFIEDTIME', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (lastModifiedTimeElement) {
                var lastModifiedTimeInstance = new Date(lastModifiedTimeElement);
                statusInfoInstance.lastModifiedTime = lastModifiedTimeInstance;
              }
              
              var queuedTimeElement = js2xml.getElement(responseDoc, statusInfoElement, 'QUEUEDTIME', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (queuedTimeElement) {
                var queuedTimeInstance = new Date(queuedTimeElement);
                statusInfoInstance.queuedTime = queuedTimeInstance;
              }
              
              var requestIdElement = js2xml.getElement(responseDoc, statusInfoElement, 'REQUESTID', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (requestIdElement) {
                var requestIdInstance = requestIdElement;
                statusInfoInstance.requestId = requestIdInstance;
              }
              
              var requestTypeElement = js2xml.getElement(responseDoc, statusInfoElement, 'REQUESTTYPE', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (requestTypeElement) {
                var requestTypeInstance = requestTypeElement;
                statusInfoInstance.requestType = requestTypeInstance;
              }
              
              var serverNameElement = js2xml.getElement(responseDoc, statusInfoElement, 'SERVERNAME', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (serverNameElement) {
                var serverNameInstance = serverNameElement;
                statusInfoInstance.serverName = serverNameInstance;
              }
              
              var statusElement = js2xml.getElement(responseDoc, statusInfoElement, 'STATUS', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (statusElement) {
                var statusInstance = statusElement;
                statusInfoInstance.status = statusInstance;
              }
            }
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Import DAC from Windows Azure blob storage.
   * 
   * @param {string} serverName The name of the server being imported to.
   * 
   * @param {DacImportParameters} [parametersOrCallback] Import parameters.
   * 
   * @param {string} [parametersOrCallback.azureEdition] The edition of this
   * database.
   * 
   * @param {BlobCredentialsParameter} [parametersOrCallback.blobCredentials]
   * Gets or sets credentials for getting the DAC.
   * 
   * @param {string} [parametersOrCallback.blobCredentials.uri] Gets or sets
   * the URI of the DAC file stored in Windows Azure Blob Storage.
   * 
   * @param {string} [parametersOrCallback.blobCredentials.storageAccessKey]
   * Gets or sets the key for the Windows Azure Storage account.
   * 
   * @param {ConnectionInfoParameter} [parametersOrCallback.connectionInfo]
   * Gets or sets connection information for the SQL Server Database.
   * 
   * @param {string} [parametersOrCallback.connectionInfo.databaseName] Gets or
   * sets the name of the database.
   * 
   * @param {string} [parametersOrCallback.connectionInfo.password] Gets or
   * sets the password for the database user.
   * 
   * @param {string} [parametersOrCallback.connectionInfo.serverName] Gets or
   * sets the SQL server name.
   * 
   * @param {string} [parametersOrCallback.connectionInfo.userName] Gets or
   * sets the username of the database user.
   * 
   * @param {number} [parametersOrCallback.databaseSizeInGB] Gets or sets the
   * size of this database.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  DacOperations.prototype.import = function(serverName, parametersOrCallback, callback) {
    var parameters = parametersOrCallback;
    if (_.isFunction(parameters)) {
      callback = parameters;
      parameters = null;
    }
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (parameters) {
      if (parameters.blobCredentials) {
        if (parameters.blobCredentials.storageAccessKey === null || parameters.blobCredentials.storageAccessKey === undefined) {
          return callback(new Error('parameters.blobCredentials.storageAccessKey cannot be null.'));
        }
        if (parameters.blobCredentials.uri === null || parameters.blobCredentials.uri === undefined) {
          return callback(new Error('parameters.blobCredentials.uri cannot be null.'));
        }
      }
      if (parameters.connectionInfo) {
        if (parameters.connectionInfo.databaseName === null || parameters.connectionInfo.databaseName === undefined) {
          return callback(new Error('parameters.connectionInfo.databaseName cannot be null.'));
        }
        if (parameters.connectionInfo.password === null || parameters.connectionInfo.password === undefined) {
          return callback(new Error('parameters.connectionInfo.password cannot be null.'));
        }
        if (parameters.connectionInfo.serverName === null || parameters.connectionInfo.serverName === undefined) {
          return callback(new Error('parameters.connectionInfo.serverName cannot be null.'));
        }
        if (parameters.connectionInfo.userName === null || parameters.connectionInfo.userName === undefined) {
          return callback(new Error('parameters.connectionInfo.userName cannot be null.'));
        }
      }
      if (parameters.databaseSizeInGB === null || parameters.databaseSizeInGB === undefined) {
        return callback(new Error('parameters.databaseSizeInGB cannot be null.'));
      }
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/DacOperations/Import';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    if (parameters) {
      var importInputElement = js2xml.createElement('ImportInput', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
      js2xml.addChildElement(requestDoc, importInputElement);
      
      if (parameters.azureEdition) {
        var azureEditionElement = js2xml.createElement('AzureEdition', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(azureEditionElement, parameters.azureEdition);
        js2xml.addChildElement(importInputElement, azureEditionElement);
      }
      
      if (parameters.blobCredentials) {
        var blobCredentialsElement = js2xml.createElement('BlobCredentials', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.addChildElement(importInputElement, blobCredentialsElement);
        
        var typeAttribute = js2xml.createAttribute('type', 'http://www.w3.org/2001/XMLSchema-instance');
        js2xml.setAttributeValue(typeAttribute, 'BlobStorageAccessKeyCredentials');
        js2xml.addAttribute(blobCredentialsElement, typeAttribute);
        
        var uriElement = js2xml.createElement('Uri', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(uriElement, parameters.blobCredentials.uri);
        js2xml.addChildElement(blobCredentialsElement, uriElement);
        
        var storageAccessKeyElement = js2xml.createElement('StorageAccessKey', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(storageAccessKeyElement, parameters.blobCredentials.storageAccessKey);
        js2xml.addChildElement(blobCredentialsElement, storageAccessKeyElement);
      }
      
      if (parameters.connectionInfo) {
        var connectionInfoElement = js2xml.createElement('ConnectionInfo', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.addChildElement(importInputElement, connectionInfoElement);
        
        var databaseNameElement = js2xml.createElement('DatabaseName', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(databaseNameElement, parameters.connectionInfo.databaseName);
        js2xml.addChildElement(connectionInfoElement, databaseNameElement);
        
        var passwordElement = js2xml.createElement('Password', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(passwordElement, parameters.connectionInfo.password);
        js2xml.addChildElement(connectionInfoElement, passwordElement);
        
        var serverNameElement = js2xml.createElement('ServerName', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(serverNameElement, parameters.connectionInfo.serverName);
        js2xml.addChildElement(connectionInfoElement, serverNameElement);
        
        var userNameElement = js2xml.createElement('UserName', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(userNameElement, parameters.connectionInfo.userName);
        js2xml.addChildElement(connectionInfoElement, userNameElement);
      }
      
      var databaseSizeInGBElement = js2xml.createElement('DatabaseSizeInGB', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
      js2xml.setElementValue(databaseSizeInGBElement, parameters.databaseSizeInGB.toString());
      js2xml.addChildElement(importInputElement, databaseSizeInGBElement);
    }
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var guidElement = js2xml.getElement(responseDoc, responseDoc, 'GUID', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/');
        if (guidElement) {
          result.guid = guidElement;
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  return DacOperations;
})();

var DatabaseOperationOperations = ( /** @lends DatabaseOperationOperations */ function() {
  /**
   * @class
   * The SQL Database Management API includes operations for get/stop SQL
   * Databases' operations for a subscription.
   * __NOTE__: An instance of this class is automatically created for an
   * instance of the [SqlManagementClient] {@link
   * SqlManagementClient~SqlManagementClient}.
   * See [databaseOperations] {@link
   * SqlManagementClient~SqlManagementClient#databaseOperations}.
   * Initializes a new instance of the DatabaseOperationOperations class.
   * @constructor
   * 
   * @param {SqlManagementClient} client Reference to the service client.
   */
  function DatabaseOperationOperations(client) {
    this.client = client;
  }
  
  /**
   * Returns information about one operation on a given operation Guid.
   * 
   * @param {string} serverName The name of the SQL Server on which the
   * operation was executed.
   * 
   * @param {string} operationGuid The Guid of the SQL Server database
   * operation to be obtained.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  DatabaseOperationOperations.prototype.get = function(serverName, operationGuid, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (operationGuid === null || operationGuid === undefined) {
      return callback(new Error('operationGuid cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/databaseoperations/' + operationGuid.trim();
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourceElement = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourceElement) {
          var serviceResourceInstance = {};
          result.databaseOperation = serviceResourceInstance;
          
          var nameElement = js2xml.getElement(responseDoc, serviceResourceElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement) {
            var nameInstance = nameElement;
            serviceResourceInstance.name = nameInstance;
          }
          
          var typeElement = js2xml.getElement(responseDoc, serviceResourceElement, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (typeElement) {
            var typeInstance = typeElement;
            serviceResourceInstance.type = typeInstance;
          }
          
          var stateElement = js2xml.getElement(responseDoc, serviceResourceElement, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (stateElement) {
            var stateInstance = stateElement;
            serviceResourceInstance.state = stateInstance;
          }
          
          var selfLinkElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SELFLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (selfLinkElement) {
            var selfLinkInstance = selfLinkElement;
            serviceResourceInstance.selfLink = selfLinkInstance;
          }
          
          var parentLinkElement = js2xml.getElement(responseDoc, serviceResourceElement, 'PARENTLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (parentLinkElement) {
            var parentLinkInstance = parentLinkElement;
            serviceResourceInstance.parentLink = parentLinkInstance;
          }
          
          var idElement = js2xml.getElement(responseDoc, serviceResourceElement, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (idElement) {
            var idInstance = idElement;
            serviceResourceInstance.id = idInstance;
          }
          
          var stateIdElement = js2xml.getElement(responseDoc, serviceResourceElement, 'STATEID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (stateIdElement) {
            var stateIdInstance = parseInt(stateIdElement, 10);
            serviceResourceInstance.stateId = stateIdInstance;
          }
          
          var sessionActivityIdElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SESSIONACTIVITYID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (sessionActivityIdElement) {
            var sessionActivityIdInstance = sessionActivityIdElement;
            serviceResourceInstance.sessionActivityId = sessionActivityIdInstance;
          }
          
          var databaseNameElement = js2xml.getElement(responseDoc, serviceResourceElement, 'DATABASENAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (databaseNameElement) {
            var databaseNameInstance = databaseNameElement;
            serviceResourceInstance.databaseName = databaseNameInstance;
          }
          
          var percentCompleteElement = js2xml.getElement(responseDoc, serviceResourceElement, 'PERCENTCOMPLETE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (percentCompleteElement) {
            var percentCompleteInstance = parseInt(percentCompleteElement, 10);
            serviceResourceInstance.percentComplete = percentCompleteInstance;
          }
          
          var errorCodeElement = js2xml.getElement(responseDoc, serviceResourceElement, 'ERRORCODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (errorCodeElement) {
            var errorCodeInstance = parseInt(errorCodeElement, 10);
            serviceResourceInstance.errorCode = errorCodeInstance;
          }
          
          var errorElement = js2xml.getElement(responseDoc, serviceResourceElement, 'ERROR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (errorElement) {
            var errorInstance = errorElement;
            serviceResourceInstance.error = errorInstance;
          }
          
          var errorSeverityElement = js2xml.getElement(responseDoc, serviceResourceElement, 'ERRORSEVERITY', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (errorSeverityElement) {
            var errorSeverityInstance = parseInt(errorSeverityElement, 10);
            serviceResourceInstance.errorSeverity = errorSeverityInstance;
          }
          
          var errorStateElement = js2xml.getElement(responseDoc, serviceResourceElement, 'ERRORSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (errorStateElement) {
            var errorStateInstance = parseInt(errorStateElement, 10);
            serviceResourceInstance.errorState = errorStateInstance;
          }
          
          var startTimeElement = js2xml.getElement(responseDoc, serviceResourceElement, 'STARTTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (startTimeElement) {
            var startTimeInstance = new Date(startTimeElement);
            serviceResourceInstance.startTime = startTimeInstance;
          }
          
          var lastModifyTimeElement = js2xml.getElement(responseDoc, serviceResourceElement, 'LASTMODIFYTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (lastModifyTimeElement) {
            var lastModifyTimeInstance = new Date(lastModifyTimeElement);
            serviceResourceInstance.lastModifyTime = lastModifyTimeInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Returns the list database operations for a given server and database.
   * 
   * @param {string} serverName The name of the SQL Server to be queried.
   * 
   * @param {string} databaseName The name of the Database to be queried.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  DatabaseOperationOperations.prototype.listByDatabase = function(serverName, databaseName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (databaseName === null || databaseName === undefined) {
      return callback(new Error('databaseName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/databaseoperations?';
    url2 = url2 + 'databaseName=' + encodeURIComponent(databaseName.trim());
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { databaseOperations: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourcesSequenceElement = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourcesSequenceElement) {
          for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, serviceResourcesSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
            var serviceResourcesElement = js2xml.getElements(responseDoc, serviceResourcesSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
            var serviceResourceInstance = {};
            result.databaseOperations.push(serviceResourceInstance);
            
            var nameElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (nameElement) {
              var nameInstance = nameElement;
              serviceResourceInstance.name = nameInstance;
            }
            
            var typeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (typeElement) {
              var typeInstance = typeElement;
              serviceResourceInstance.type = typeInstance;
            }
            
            var stateElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (stateElement) {
              var stateInstance = stateElement;
              serviceResourceInstance.state = stateInstance;
            }
            
            var selfLinkElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SELFLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (selfLinkElement) {
              var selfLinkInstance = selfLinkElement;
              serviceResourceInstance.selfLink = selfLinkInstance;
            }
            
            var parentLinkElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'PARENTLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (parentLinkElement) {
              var parentLinkInstance = parentLinkElement;
              serviceResourceInstance.parentLink = parentLinkInstance;
            }
            
            var idElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (idElement) {
              var idInstance = idElement;
              serviceResourceInstance.id = idInstance;
            }
            
            var stateIdElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'STATEID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (stateIdElement) {
              var stateIdInstance = parseInt(stateIdElement, 10);
              serviceResourceInstance.stateId = stateIdInstance;
            }
            
            var sessionActivityIdElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SESSIONACTIVITYID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (sessionActivityIdElement) {
              var sessionActivityIdInstance = sessionActivityIdElement;
              serviceResourceInstance.sessionActivityId = sessionActivityIdInstance;
            }
            
            var databaseNameElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'DATABASENAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (databaseNameElement) {
              var databaseNameInstance = databaseNameElement;
              serviceResourceInstance.databaseName = databaseNameInstance;
            }
            
            var percentCompleteElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'PERCENTCOMPLETE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (percentCompleteElement) {
              var percentCompleteInstance = parseInt(percentCompleteElement, 10);
              serviceResourceInstance.percentComplete = percentCompleteInstance;
            }
            
            var errorCodeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ERRORCODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (errorCodeElement) {
              var errorCodeInstance = parseInt(errorCodeElement, 10);
              serviceResourceInstance.errorCode = errorCodeInstance;
            }
            
            var errorElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ERROR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (errorElement) {
              var errorInstance = errorElement;
              serviceResourceInstance.error = errorInstance;
            }
            
            var errorSeverityElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ERRORSEVERITY', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (errorSeverityElement) {
              var errorSeverityInstance = parseInt(errorSeverityElement, 10);
              serviceResourceInstance.errorSeverity = errorSeverityInstance;
            }
            
            var errorStateElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ERRORSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (errorStateElement) {
              var errorStateInstance = parseInt(errorStateElement, 10);
              serviceResourceInstance.errorState = errorStateInstance;
            }
            
            var startTimeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'STARTTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (startTimeElement) {
              var startTimeInstance = new Date(startTimeElement);
              serviceResourceInstance.startTime = startTimeInstance;
            }
            
            var lastModifyTimeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'LASTMODIFYTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (lastModifyTimeElement) {
              var lastModifyTimeInstance = new Date(lastModifyTimeElement);
              serviceResourceInstance.lastModifyTime = lastModifyTimeInstance;
            }
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Returns the list database operations for a given server.
   * 
   * @param {string} serverName The name of the SQL Server to be queried.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  DatabaseOperationOperations.prototype.listByServer = function(serverName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/databaseoperations?contentview=generic';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { databaseOperations: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourcesSequenceElement = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourcesSequenceElement) {
          for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, serviceResourcesSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
            var serviceResourcesElement = js2xml.getElements(responseDoc, serviceResourcesSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
            var serviceResourceInstance = {};
            result.databaseOperations.push(serviceResourceInstance);
            
            var nameElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (nameElement) {
              var nameInstance = nameElement;
              serviceResourceInstance.name = nameInstance;
            }
            
            var typeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (typeElement) {
              var typeInstance = typeElement;
              serviceResourceInstance.type = typeInstance;
            }
            
            var stateElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (stateElement) {
              var stateInstance = stateElement;
              serviceResourceInstance.state = stateInstance;
            }
            
            var selfLinkElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SELFLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (selfLinkElement) {
              var selfLinkInstance = selfLinkElement;
              serviceResourceInstance.selfLink = selfLinkInstance;
            }
            
            var parentLinkElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'PARENTLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (parentLinkElement) {
              var parentLinkInstance = parentLinkElement;
              serviceResourceInstance.parentLink = parentLinkInstance;
            }
            
            var idElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (idElement) {
              var idInstance = idElement;
              serviceResourceInstance.id = idInstance;
            }
            
            var stateIdElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'STATEID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (stateIdElement) {
              var stateIdInstance = parseInt(stateIdElement, 10);
              serviceResourceInstance.stateId = stateIdInstance;
            }
            
            var sessionActivityIdElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SESSIONACTIVITYID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (sessionActivityIdElement) {
              var sessionActivityIdInstance = sessionActivityIdElement;
              serviceResourceInstance.sessionActivityId = sessionActivityIdInstance;
            }
            
            var databaseNameElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'DATABASENAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (databaseNameElement) {
              var databaseNameInstance = databaseNameElement;
              serviceResourceInstance.databaseName = databaseNameInstance;
            }
            
            var percentCompleteElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'PERCENTCOMPLETE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (percentCompleteElement) {
              var percentCompleteInstance = parseInt(percentCompleteElement, 10);
              serviceResourceInstance.percentComplete = percentCompleteInstance;
            }
            
            var errorCodeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ERRORCODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (errorCodeElement) {
              var errorCodeInstance = parseInt(errorCodeElement, 10);
              serviceResourceInstance.errorCode = errorCodeInstance;
            }
            
            var errorElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ERROR', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (errorElement) {
              var errorInstance = errorElement;
              serviceResourceInstance.error = errorInstance;
            }
            
            var errorSeverityElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ERRORSEVERITY', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (errorSeverityElement) {
              var errorSeverityInstance = parseInt(errorSeverityElement, 10);
              serviceResourceInstance.errorSeverity = errorSeverityInstance;
            }
            
            var errorStateElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ERRORSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (errorStateElement) {
              var errorStateInstance = parseInt(errorStateElement, 10);
              serviceResourceInstance.errorState = errorStateInstance;
            }
            
            var startTimeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'STARTTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (startTimeElement) {
              var startTimeInstance = new Date(startTimeElement);
              serviceResourceInstance.startTime = startTimeInstance;
            }
            
            var lastModifyTimeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'LASTMODIFYTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (lastModifyTimeElement) {
              var lastModifyTimeInstance = new Date(lastModifyTimeElement);
              serviceResourceInstance.lastModifyTime = lastModifyTimeInstance;
            }
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  return DatabaseOperationOperations;
})();

var DatabaseOperations = ( /** @lends DatabaseOperations */ function() {
  /**
   * @class
   * The SQL Database Management API includes operations for managing SQL
   * Databases for a subscription.
   * __NOTE__: An instance of this class is automatically created for an
   * instance of the [SqlManagementClient] {@link
   * SqlManagementClient~SqlManagementClient}.
   * See [databases] {@link SqlManagementClient~SqlManagementClient#databases}.
   * Initializes a new instance of the DatabaseOperations class.
   * @constructor
   * 
   * @param {SqlManagementClient} client Reference to the service client.
   */
  function DatabaseOperations(client) {
    this.client = client;
  }
  
  /**
   * Creates a database in a SQL Server database server.
   * 
   * @param {string} serverName The name of the SQL Server where the database
   * will be created.
   * 
   * @param {DatabaseCreateParameters} parameters The parameters for the create
   * database operation.
   * 
   * @param {string} parameters.name Gets or sets the name for the new database.
   * 
   * @param {string} parameters.edition Gets or sets the edition for the new
   * database.
   * 
   * @param {number} parameters.maximumDatabaseSizeInGB Gets or sets the
   * maximum size of this database, in Gigabytes.
   * 
   * @param {string} [parameters.collationName] Gets or sets the collation name
   * for the new database.
   * 
   * @param {string} [parameters.serviceObjectiveId] Gets or sets the id of
   * this service objective.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  DatabaseOperations.prototype.create = function(serverName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.edition === null || parameters.edition === undefined) {
      return callback(new Error('parameters.edition cannot be null.'));
    }
    if (parameters.maximumDatabaseSizeInGB === null || parameters.maximumDatabaseSizeInGB === undefined) {
      return callback(new Error('parameters.maximumDatabaseSizeInGB cannot be null.'));
    }
    if (parameters.name === null || parameters.name === undefined) {
      return callback(new Error('parameters.name cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/databases';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var serviceResourceElement = js2xml.createElement('ServiceResource', 'http://schemas.microsoft.com/windowsazure');
    js2xml.addChildElement(requestDoc, serviceResourceElement);
    
    var nameElement = js2xml.createElement('Name', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(nameElement, parameters.name);
    js2xml.addChildElement(serviceResourceElement, nameElement);
    
    var editionElement = js2xml.createElement('Edition', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(editionElement, parameters.edition);
    js2xml.addChildElement(serviceResourceElement, editionElement);
    
    var maxSizeGBElement = js2xml.createElement('MaxSizeGB', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(maxSizeGBElement, parameters.maximumDatabaseSizeInGB.toString());
    js2xml.addChildElement(serviceResourceElement, maxSizeGBElement);
    
    if (parameters.collationName) {
      var collationNameElement = js2xml.createElement('CollationName', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(collationNameElement, parameters.collationName);
      js2xml.addChildElement(serviceResourceElement, collationNameElement);
    }
    
    if (parameters.serviceObjectiveId) {
      var serviceObjectiveIdElement = js2xml.createElement('ServiceObjectiveId', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(serviceObjectiveIdElement, parameters.serviceObjectiveId);
      js2xml.addChildElement(serviceResourceElement, serviceObjectiveIdElement);
    }
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 201) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourceElement2 = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourceElement2) {
          var serviceResourceInstance = {};
          result.database = serviceResourceInstance;
          
          var nameElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement2) {
            var nameInstance = nameElement2;
            serviceResourceInstance.name = nameInstance;
          }
          
          var idElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (idElement) {
            var idInstance = parseInt(idElement, 10);
            serviceResourceInstance.id = idInstance;
          }
          
          var typeElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (typeElement) {
            var typeInstance = typeElement;
            serviceResourceInstance.type = typeInstance;
          }
          
          var stateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (stateElement) {
            var stateInstance = stateElement;
            serviceResourceInstance.state = stateInstance;
          }
          
          var editionElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'EDITION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (editionElement2) {
            var editionInstance = editionElement2;
            serviceResourceInstance.edition = editionInstance;
          }
          
          var maxSizeGBElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'MAXSIZEGB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (maxSizeGBElement2) {
            var maxSizeGBInstance = parseInt(maxSizeGBElement2, 10);
            serviceResourceInstance.maximumDatabaseSizeInGB = maxSizeGBInstance;
          }
          
          var collationNameElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'COLLATIONNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (collationNameElement2) {
            var collationNameInstance = collationNameElement2;
            serviceResourceInstance.collationName = collationNameInstance;
          }
          
          var creationDateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'CREATIONDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (creationDateElement) {
            var creationDateInstance = new Date(creationDateElement);
            serviceResourceInstance.creationDate = creationDateInstance;
          }
          
          var isFederationRootElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'ISFEDERATIONROOT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (isFederationRootElement) {
            var isFederationRootInstance = isFederationRootElement === 'true';
            serviceResourceInstance.isFederationRoot = isFederationRootInstance;
          }
          
          var isSystemObjectElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'ISSYSTEMOBJECT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (isSystemObjectElement) {
            var isSystemObjectInstance = isSystemObjectElement === 'true';
            serviceResourceInstance.isSystemObject = isSystemObjectInstance;
          }
          
          var sizeMBElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SIZEMB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (sizeMBElement) {
            var sizeMBInstance = sizeMBElement;
            serviceResourceInstance.sizeMB = sizeMBInstance;
          }
          
          var serviceObjectiveAssignmentErrorCodeElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTERRORCODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentErrorCodeElement) {
            var serviceObjectiveAssignmentErrorCodeInstance = serviceObjectiveAssignmentErrorCodeElement;
            serviceResourceInstance.serviceObjectiveAssignmentErrorCode = serviceObjectiveAssignmentErrorCodeInstance;
          }
          
          var serviceObjectiveAssignmentErrorDescriptionElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTERRORDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentErrorDescriptionElement) {
            var serviceObjectiveAssignmentErrorDescriptionInstance = serviceObjectiveAssignmentErrorDescriptionElement;
            serviceResourceInstance.serviceObjectiveAssignmentErrorDescription = serviceObjectiveAssignmentErrorDescriptionInstance;
          }
          
          var serviceObjectiveAssignmentStateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentStateElement) {
            var serviceObjectiveAssignmentStateInstance = serviceObjectiveAssignmentStateElement;
            serviceResourceInstance.serviceObjectiveAssignmentState = serviceObjectiveAssignmentStateInstance;
          }
          
          var serviceObjectiveAssignmentStateDescriptionElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTSTATEDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentStateDescriptionElement) {
            var serviceObjectiveAssignmentStateDescriptionInstance = serviceObjectiveAssignmentStateDescriptionElement;
            serviceResourceInstance.serviceObjectiveAssignmentStateDescription = serviceObjectiveAssignmentStateDescriptionInstance;
          }
          
          var serviceObjectiveAssignmentSuccessDateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTSUCCESSDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentSuccessDateElement) {
            var serviceObjectiveAssignmentSuccessDateInstance = serviceObjectiveAssignmentSuccessDateElement;
            serviceResourceInstance.serviceObjectiveAssignmentSuccessDate = serviceObjectiveAssignmentSuccessDateInstance;
          }
          
          var serviceObjectiveIdElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveIdElement2) {
            var serviceObjectiveIdInstance = serviceObjectiveIdElement2;
            serviceResourceInstance.serviceObjectiveId = serviceObjectiveIdInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Drops a SQL Database server from a subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715285.aspx for
   * more information)
   * 
   * @param {string} serverName The name of the server on which the database is
   * found.
   * 
   * @param {string} databaseName The name of the database to be deleted.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  DatabaseOperations.prototype.delete = function(serverName, databaseName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (databaseName === null || databaseName === undefined) {
      return callback(new Error('databaseName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/databases/' + databaseName.trim();
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'DELETE';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Returns information about a SQL Server database.
   * 
   * @param {string} serverName The name of the SQL Server on which the
   * database is housed.
   * 
   * @param {string} databaseName The name of the SQL Server database to be
   * obtained.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  DatabaseOperations.prototype.get = function(serverName, databaseName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (databaseName === null || databaseName === undefined) {
      return callback(new Error('databaseName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/databases/' + databaseName.trim();
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourceElement = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourceElement) {
          var serviceResourceInstance = {};
          result.database = serviceResourceInstance;
          
          var nameElement = js2xml.getElement(responseDoc, serviceResourceElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement) {
            var nameInstance = nameElement;
            serviceResourceInstance.name = nameInstance;
          }
          
          var idElement = js2xml.getElement(responseDoc, serviceResourceElement, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (idElement) {
            var idInstance = parseInt(idElement, 10);
            serviceResourceInstance.id = idInstance;
          }
          
          var typeElement = js2xml.getElement(responseDoc, serviceResourceElement, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (typeElement) {
            var typeInstance = typeElement;
            serviceResourceInstance.type = typeInstance;
          }
          
          var stateElement = js2xml.getElement(responseDoc, serviceResourceElement, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (stateElement) {
            var stateInstance = stateElement;
            serviceResourceInstance.state = stateInstance;
          }
          
          var editionElement = js2xml.getElement(responseDoc, serviceResourceElement, 'EDITION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (editionElement) {
            var editionInstance = editionElement;
            serviceResourceInstance.edition = editionInstance;
          }
          
          var maxSizeGBElement = js2xml.getElement(responseDoc, serviceResourceElement, 'MAXSIZEGB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (maxSizeGBElement) {
            var maxSizeGBInstance = parseInt(maxSizeGBElement, 10);
            serviceResourceInstance.maximumDatabaseSizeInGB = maxSizeGBInstance;
          }
          
          var collationNameElement = js2xml.getElement(responseDoc, serviceResourceElement, 'COLLATIONNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (collationNameElement) {
            var collationNameInstance = collationNameElement;
            serviceResourceInstance.collationName = collationNameInstance;
          }
          
          var creationDateElement = js2xml.getElement(responseDoc, serviceResourceElement, 'CREATIONDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (creationDateElement) {
            var creationDateInstance = new Date(creationDateElement);
            serviceResourceInstance.creationDate = creationDateInstance;
          }
          
          var isFederationRootElement = js2xml.getElement(responseDoc, serviceResourceElement, 'ISFEDERATIONROOT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (isFederationRootElement) {
            var isFederationRootInstance = isFederationRootElement === 'true';
            serviceResourceInstance.isFederationRoot = isFederationRootInstance;
          }
          
          var isSystemObjectElement = js2xml.getElement(responseDoc, serviceResourceElement, 'ISSYSTEMOBJECT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (isSystemObjectElement) {
            var isSystemObjectInstance = isSystemObjectElement === 'true';
            serviceResourceInstance.isSystemObject = isSystemObjectInstance;
          }
          
          var sizeMBElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SIZEMB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (sizeMBElement) {
            var sizeMBInstance = sizeMBElement;
            serviceResourceInstance.sizeMB = sizeMBInstance;
          }
          
          var serviceObjectiveAssignmentErrorCodeElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SERVICEOBJECTIVEASSIGNMENTERRORCODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentErrorCodeElement) {
            var serviceObjectiveAssignmentErrorCodeInstance = serviceObjectiveAssignmentErrorCodeElement;
            serviceResourceInstance.serviceObjectiveAssignmentErrorCode = serviceObjectiveAssignmentErrorCodeInstance;
          }
          
          var serviceObjectiveAssignmentErrorDescriptionElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SERVICEOBJECTIVEASSIGNMENTERRORDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentErrorDescriptionElement) {
            var serviceObjectiveAssignmentErrorDescriptionInstance = serviceObjectiveAssignmentErrorDescriptionElement;
            serviceResourceInstance.serviceObjectiveAssignmentErrorDescription = serviceObjectiveAssignmentErrorDescriptionInstance;
          }
          
          var serviceObjectiveAssignmentStateElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SERVICEOBJECTIVEASSIGNMENTSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentStateElement) {
            var serviceObjectiveAssignmentStateInstance = serviceObjectiveAssignmentStateElement;
            serviceResourceInstance.serviceObjectiveAssignmentState = serviceObjectiveAssignmentStateInstance;
          }
          
          var serviceObjectiveAssignmentStateDescriptionElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SERVICEOBJECTIVEASSIGNMENTSTATEDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentStateDescriptionElement) {
            var serviceObjectiveAssignmentStateDescriptionInstance = serviceObjectiveAssignmentStateDescriptionElement;
            serviceResourceInstance.serviceObjectiveAssignmentStateDescription = serviceObjectiveAssignmentStateDescriptionInstance;
          }
          
          var serviceObjectiveAssignmentSuccessDateElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SERVICEOBJECTIVEASSIGNMENTSUCCESSDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentSuccessDateElement) {
            var serviceObjectiveAssignmentSuccessDateInstance = serviceObjectiveAssignmentSuccessDateElement;
            serviceResourceInstance.serviceObjectiveAssignmentSuccessDate = serviceObjectiveAssignmentSuccessDateInstance;
          }
          
          var serviceObjectiveIdElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SERVICEOBJECTIVEID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveIdElement) {
            var serviceObjectiveIdInstance = serviceObjectiveIdElement;
            serviceResourceInstance.serviceObjectiveId = serviceObjectiveIdInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Returns information about a SQL Server database event logs.
   * 
   * @param {string} serverName The name of the SQL Server on which the
   * database is housed.
   * 
   * @param {string} databaseName The name of the SQL Server database to be
   * obtained.
   * 
   * @param {DatabaseGetEventLogsParameters} parameters The parameters for the
   * get event logs database operation.
   * 
   * @param {date} parameters.startDate Gets or sets the start date.
   * 
   * @param {number} parameters.intervalSizeInMinutes Gets or sets the interval
   * size in minutes.
   * 
   * @param {string} parameters.eventTypes Gets or sets the event types.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  DatabaseOperations.prototype.getEventLogs = function(serverName, databaseName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (databaseName === null || databaseName === undefined) {
      return callback(new Error('databaseName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.eventTypes === null || parameters.eventTypes === undefined) {
      return callback(new Error('parameters.eventTypes cannot be null.'));
    }
    if (parameters.intervalSizeInMinutes === null || parameters.intervalSizeInMinutes === undefined) {
      return callback(new Error('parameters.intervalSizeInMinutes cannot be null.'));
    }
    if (parameters.startDate === null || parameters.startDate === undefined) {
      return callback(new Error('parameters.startDate cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/databases/' + databaseName.trim() + '/events?';
    url2 = url2 + 'startDate=' + encodeURIComponent(parameters.startDate.toISOString());
    url2 = url2 + '&intervalSizeInMinutes=' + encodeURIComponent(parameters.intervalSizeInMinutes.toString());
    url2 = url2 + '&eventTypes=' + encodeURIComponent(parameters.eventTypes.trim());
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { eventLogs: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourcesSequenceElement = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourcesSequenceElement) {
          for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, serviceResourcesSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
            var serviceResourcesElement = js2xml.getElements(responseDoc, serviceResourcesSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
            var serviceResourceInstance = {};
            result.eventLogs.push(serviceResourceInstance);
            
            var nameElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (nameElement) {
              var nameInstance = nameElement;
              serviceResourceInstance.name = nameInstance;
            }
            
            var typeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (typeElement) {
              var typeInstance = typeElement;
              serviceResourceInstance.type = typeInstance;
            }
            
            var stateElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (stateElement) {
              var stateInstance = stateElement;
              serviceResourceInstance.state = stateInstance;
            }
            
            var selfLinkElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SELFLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (selfLinkElement) {
              var selfLinkInstance = selfLinkElement;
              serviceResourceInstance.selfLink = selfLinkInstance;
            }
            
            var parentLinkElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'PARENTLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (parentLinkElement) {
              var parentLinkInstance = parentLinkElement;
              serviceResourceInstance.parentLink = parentLinkInstance;
            }
            
            var databaseNameElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'DATABASENAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (databaseNameElement) {
              var databaseNameInstance = databaseNameElement;
              serviceResourceInstance.databaseName = databaseNameInstance;
            }
            
            var startTimeUtcElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'STARTTIMEUTC', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (startTimeUtcElement) {
              var startTimeUtcInstance = new Date(startTimeUtcElement);
              serviceResourceInstance.startTimeUtc = startTimeUtcInstance;
            }
            
            var intervalSizeInMinutesElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'INTERVALSIZEINMINUTES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (intervalSizeInMinutesElement) {
              var intervalSizeInMinutesInstance = parseInt(intervalSizeInMinutesElement, 10);
              serviceResourceInstance.intervalSizeInMinutes = intervalSizeInMinutesInstance;
            }
            
            var eventCategoryElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'EVENTCATEGORY', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (eventCategoryElement) {
              var eventCategoryInstance = eventCategoryElement;
              serviceResourceInstance.eventCategory = eventCategoryInstance;
            }
            
            var eventTypeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'EVENTTYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (eventTypeElement) {
              var eventTypeInstance = eventTypeElement;
              serviceResourceInstance.eventType = eventTypeInstance;
            }
            
            var eventSubtypeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'EVENTSUBTYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (eventSubtypeElement) {
              var eventSubtypeInstance = eventSubtypeElement;
              serviceResourceInstance.eventSubtype = eventSubtypeInstance;
            }
            
            var eventSubtypeDescriptionElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'EVENTSUBTYPEDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (eventSubtypeDescriptionElement) {
              var eventSubtypeDescriptionInstance = eventSubtypeDescriptionElement;
              serviceResourceInstance.eventSubtypeDescription = eventSubtypeDescriptionInstance;
            }
            
            var numberOfEventsElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'NUMBEROFEVENTS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (numberOfEventsElement) {
              var numberOfEventsInstance = parseInt(numberOfEventsElement, 10);
              serviceResourceInstance.numberOfEvents = numberOfEventsInstance;
            }
            
            var severityElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SEVERITY', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (severityElement) {
              var severityInstance = parseInt(severityElement, 10);
              serviceResourceInstance.severity = severityInstance;
            }
            
            var descriptionElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'DESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (descriptionElement) {
              var descriptionInstance = descriptionElement;
              serviceResourceInstance.description = descriptionInstance;
            }
            
            var additionalDataElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ADDITIONALDATA', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (additionalDataElement) {
              var isNil = false;
              var nilAttribute = js2xml.getAttribute(responseDoc, additionalDataElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
              if (nilAttribute) {
                isNil = nilAttribute === 'true';
              }
              if (isNil === false) {
                var additionalDataInstance = additionalDataElement;
                serviceResourceInstance.additionalData = additionalDataInstance;
              }
            }
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Returns the list SQL Server databases.
   * 
   * @param {string} serverName The name of the database server to be queried.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  DatabaseOperations.prototype.list = function(serverName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/databases?contentview=generic';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { databases: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourcesSequenceElement = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourcesSequenceElement) {
          for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, serviceResourcesSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
            var serviceResourcesElement = js2xml.getElements(responseDoc, serviceResourcesSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
            var serviceResourceInstance = {};
            result.databases.push(serviceResourceInstance);
            
            var nameElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (nameElement) {
              var nameInstance = nameElement;
              serviceResourceInstance.name = nameInstance;
            }
            
            var idElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (idElement) {
              var idInstance = parseInt(idElement, 10);
              serviceResourceInstance.id = idInstance;
            }
            
            var typeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (typeElement) {
              var typeInstance = typeElement;
              serviceResourceInstance.type = typeInstance;
            }
            
            var stateElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (stateElement) {
              var stateInstance = stateElement;
              serviceResourceInstance.state = stateInstance;
            }
            
            var editionElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'EDITION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (editionElement) {
              var editionInstance = editionElement;
              serviceResourceInstance.edition = editionInstance;
            }
            
            var maxSizeGBElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'MAXSIZEGB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (maxSizeGBElement) {
              var maxSizeGBInstance = parseInt(maxSizeGBElement, 10);
              serviceResourceInstance.maximumDatabaseSizeInGB = maxSizeGBInstance;
            }
            
            var collationNameElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'COLLATIONNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (collationNameElement) {
              var collationNameInstance = collationNameElement;
              serviceResourceInstance.collationName = collationNameInstance;
            }
            
            var creationDateElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'CREATIONDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (creationDateElement) {
              var creationDateInstance = new Date(creationDateElement);
              serviceResourceInstance.creationDate = creationDateInstance;
            }
            
            var isFederationRootElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ISFEDERATIONROOT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (isFederationRootElement) {
              var isFederationRootInstance = isFederationRootElement === 'true';
              serviceResourceInstance.isFederationRoot = isFederationRootInstance;
            }
            
            var isSystemObjectElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ISSYSTEMOBJECT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (isSystemObjectElement) {
              var isSystemObjectInstance = isSystemObjectElement === 'true';
              serviceResourceInstance.isSystemObject = isSystemObjectInstance;
            }
            
            var sizeMBElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SIZEMB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (sizeMBElement) {
              var sizeMBInstance = sizeMBElement;
              serviceResourceInstance.sizeMB = sizeMBInstance;
            }
            
            var serviceObjectiveAssignmentErrorCodeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SERVICEOBJECTIVEASSIGNMENTERRORCODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (serviceObjectiveAssignmentErrorCodeElement) {
              var serviceObjectiveAssignmentErrorCodeInstance = serviceObjectiveAssignmentErrorCodeElement;
              serviceResourceInstance.serviceObjectiveAssignmentErrorCode = serviceObjectiveAssignmentErrorCodeInstance;
            }
            
            var serviceObjectiveAssignmentErrorDescriptionElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SERVICEOBJECTIVEASSIGNMENTERRORDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (serviceObjectiveAssignmentErrorDescriptionElement) {
              var serviceObjectiveAssignmentErrorDescriptionInstance = serviceObjectiveAssignmentErrorDescriptionElement;
              serviceResourceInstance.serviceObjectiveAssignmentErrorDescription = serviceObjectiveAssignmentErrorDescriptionInstance;
            }
            
            var serviceObjectiveAssignmentStateElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SERVICEOBJECTIVEASSIGNMENTSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (serviceObjectiveAssignmentStateElement) {
              var serviceObjectiveAssignmentStateInstance = serviceObjectiveAssignmentStateElement;
              serviceResourceInstance.serviceObjectiveAssignmentState = serviceObjectiveAssignmentStateInstance;
            }
            
            var serviceObjectiveAssignmentStateDescriptionElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SERVICEOBJECTIVEASSIGNMENTSTATEDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (serviceObjectiveAssignmentStateDescriptionElement) {
              var serviceObjectiveAssignmentStateDescriptionInstance = serviceObjectiveAssignmentStateDescriptionElement;
              serviceResourceInstance.serviceObjectiveAssignmentStateDescription = serviceObjectiveAssignmentStateDescriptionInstance;
            }
            
            var serviceObjectiveAssignmentSuccessDateElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SERVICEOBJECTIVEASSIGNMENTSUCCESSDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (serviceObjectiveAssignmentSuccessDateElement) {
              var serviceObjectiveAssignmentSuccessDateInstance = serviceObjectiveAssignmentSuccessDateElement;
              serviceResourceInstance.serviceObjectiveAssignmentSuccessDate = serviceObjectiveAssignmentSuccessDateInstance;
            }
            
            var serviceObjectiveIdElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SERVICEOBJECTIVEID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (serviceObjectiveIdElement) {
              var serviceObjectiveIdInstance = serviceObjectiveIdElement;
              serviceResourceInstance.serviceObjectiveId = serviceObjectiveIdInstance;
            }
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Updates SQL Server database information.
   * 
   * @param {string} serverName The name of the SQL Server where the database
   * is housed.
   * 
   * @param {string} databaseName The name of the SQL Server database to be
   * obtained.
   * 
   * @param {DatabaseUpdateParameters} parameters The parameters for the update
   * database operation.
   * 
   * @param {string} [parameters.name] Gets or sets the name of the database.
   * 
   * @param {number} parameters.id Gets or sets the id of the database.
   * 
   * @param {string} parameters.edition Gets or sets the edition for the new
   * database.
   * 
   * @param {number} parameters.maximumDatabaseSizeInGB Gets or sets the
   * maximum size of this database, in Gigabytes.
   * 
   * @param {string} parameters.collationName Gets or sets the collation name
   * for the new database.
   * 
   * @param {string} [parameters.serviceObjectiveId] Gets or sets the id of
   * this service objective.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  DatabaseOperations.prototype.update = function(serverName, databaseName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (databaseName === null || databaseName === undefined) {
      return callback(new Error('databaseName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.collationName === null || parameters.collationName === undefined) {
      return callback(new Error('parameters.collationName cannot be null.'));
    }
    if (parameters.edition === null || parameters.edition === undefined) {
      return callback(new Error('parameters.edition cannot be null.'));
    }
    if (parameters.id === null || parameters.id === undefined) {
      return callback(new Error('parameters.id cannot be null.'));
    }
    if (parameters.maximumDatabaseSizeInGB === null || parameters.maximumDatabaseSizeInGB === undefined) {
      return callback(new Error('parameters.maximumDatabaseSizeInGB cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/databases/' + databaseName.trim();
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'PUT';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var serviceResourceElement = js2xml.createElement('ServiceResource', 'http://schemas.microsoft.com/windowsazure');
    js2xml.addChildElement(requestDoc, serviceResourceElement);
    
    if (parameters.name) {
      var nameElement = js2xml.createElement('Name', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(nameElement, parameters.name);
      js2xml.addChildElement(serviceResourceElement, nameElement);
    }
    
    var idElement = js2xml.createElement('Id', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(idElement, parameters.id.toString());
    js2xml.addChildElement(serviceResourceElement, idElement);
    
    var editionElement = js2xml.createElement('Edition', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(editionElement, parameters.edition);
    js2xml.addChildElement(serviceResourceElement, editionElement);
    
    var maxSizeGBElement = js2xml.createElement('MaxSizeGB', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(maxSizeGBElement, parameters.maximumDatabaseSizeInGB.toString());
    js2xml.addChildElement(serviceResourceElement, maxSizeGBElement);
    
    var collationNameElement = js2xml.createElement('CollationName', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(collationNameElement, parameters.collationName);
    js2xml.addChildElement(serviceResourceElement, collationNameElement);
    
    if (parameters.serviceObjectiveId) {
      var serviceObjectiveIdElement = js2xml.createElement('ServiceObjectiveId', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(serviceObjectiveIdElement, parameters.serviceObjectiveId);
      js2xml.addChildElement(serviceResourceElement, serviceObjectiveIdElement);
    }
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourceElement2 = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourceElement2) {
          var serviceResourceInstance = {};
          result.database = serviceResourceInstance;
          
          var nameElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement2) {
            var nameInstance = nameElement2;
            serviceResourceInstance.name = nameInstance;
          }
          
          var idElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (idElement2) {
            var idInstance = parseInt(idElement2, 10);
            serviceResourceInstance.id = idInstance;
          }
          
          var typeElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (typeElement) {
            var typeInstance = typeElement;
            serviceResourceInstance.type = typeInstance;
          }
          
          var stateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (stateElement) {
            var stateInstance = stateElement;
            serviceResourceInstance.state = stateInstance;
          }
          
          var editionElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'EDITION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (editionElement2) {
            var editionInstance = editionElement2;
            serviceResourceInstance.edition = editionInstance;
          }
          
          var maxSizeGBElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'MAXSIZEGB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (maxSizeGBElement2) {
            var maxSizeGBInstance = parseInt(maxSizeGBElement2, 10);
            serviceResourceInstance.maximumDatabaseSizeInGB = maxSizeGBInstance;
          }
          
          var collationNameElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'COLLATIONNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (collationNameElement2) {
            var collationNameInstance = collationNameElement2;
            serviceResourceInstance.collationName = collationNameInstance;
          }
          
          var creationDateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'CREATIONDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (creationDateElement) {
            var creationDateInstance = new Date(creationDateElement);
            serviceResourceInstance.creationDate = creationDateInstance;
          }
          
          var isFederationRootElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'ISFEDERATIONROOT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (isFederationRootElement) {
            var isFederationRootInstance = isFederationRootElement === 'true';
            serviceResourceInstance.isFederationRoot = isFederationRootInstance;
          }
          
          var isSystemObjectElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'ISSYSTEMOBJECT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (isSystemObjectElement) {
            var isSystemObjectInstance = isSystemObjectElement === 'true';
            serviceResourceInstance.isSystemObject = isSystemObjectInstance;
          }
          
          var sizeMBElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SIZEMB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (sizeMBElement) {
            var sizeMBInstance = sizeMBElement;
            serviceResourceInstance.sizeMB = sizeMBInstance;
          }
          
          var serviceObjectiveAssignmentErrorCodeElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTERRORCODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentErrorCodeElement) {
            var serviceObjectiveAssignmentErrorCodeInstance = serviceObjectiveAssignmentErrorCodeElement;
            serviceResourceInstance.serviceObjectiveAssignmentErrorCode = serviceObjectiveAssignmentErrorCodeInstance;
          }
          
          var serviceObjectiveAssignmentErrorDescriptionElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTERRORDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentErrorDescriptionElement) {
            var serviceObjectiveAssignmentErrorDescriptionInstance = serviceObjectiveAssignmentErrorDescriptionElement;
            serviceResourceInstance.serviceObjectiveAssignmentErrorDescription = serviceObjectiveAssignmentErrorDescriptionInstance;
          }
          
          var serviceObjectiveAssignmentStateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentStateElement) {
            var serviceObjectiveAssignmentStateInstance = serviceObjectiveAssignmentStateElement;
            serviceResourceInstance.serviceObjectiveAssignmentState = serviceObjectiveAssignmentStateInstance;
          }
          
          var serviceObjectiveAssignmentStateDescriptionElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTSTATEDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentStateDescriptionElement) {
            var serviceObjectiveAssignmentStateDescriptionInstance = serviceObjectiveAssignmentStateDescriptionElement;
            serviceResourceInstance.serviceObjectiveAssignmentStateDescription = serviceObjectiveAssignmentStateDescriptionInstance;
          }
          
          var serviceObjectiveAssignmentSuccessDateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTSUCCESSDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentSuccessDateElement) {
            var serviceObjectiveAssignmentSuccessDateInstance = serviceObjectiveAssignmentSuccessDateElement;
            serviceResourceInstance.serviceObjectiveAssignmentSuccessDate = serviceObjectiveAssignmentSuccessDateInstance;
          }
          
          var serviceObjectiveIdElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveIdElement2) {
            var serviceObjectiveIdInstance = serviceObjectiveIdElement2;
            serviceResourceInstance.serviceObjectiveId = serviceObjectiveIdInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  return DatabaseOperations;
})();

var FirewallRuleOperations = ( /** @lends FirewallRuleOperations */ function() {
  /**
   * @class
   * The Windows Azure SQL Database Management API includes operations for
   * managing the server-level firewall rules for SQL Database servers.You
   * cannot manage the database-level firewall rules using the Windows Azure
   * SQL Database Management API; they can only be managed by running the
   * Transact-SQL statements against the master or individual user databases.
   * (see http://msdn.microsoft.com/en-us/library/windowsazure/gg715276.aspx
   * for more information)
   * __NOTE__: An instance of this class is automatically created for an
   * instance of the [SqlManagementClient] {@link
   * SqlManagementClient~SqlManagementClient}.
   * See [firewallRules] {@link
   * SqlManagementClient~SqlManagementClient#firewallRules}.
   * Initializes a new instance of the FirewallRuleOperations class.
   * @constructor
   * 
   * @param {SqlManagementClient} client Reference to the service client.
   */
  function FirewallRuleOperations(client) {
    this.client = client;
  }
  
  /**
   * Adds a new server-level firewall rule for a SQL Database server that
   * belongs to a subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715280.aspx for
   * more information)
   * 
   * @param {string} serverName The name of the SQL database server to which
   * this rule will be applied.
   * 
   * @param {FirewallRuleCreateParameters} parameters Parameters for the Create
   * Firewall Rule operation.
   * 
   * @param {string} parameters.name Gets or sets the name of this firewall
   * rule.
   * 
   * @param {string} parameters.startIPAddress Gets or sets the beginning IP
   * address applied to this firewall rule.
   * 
   * @param {string} parameters.endIPAddress Gets or sets the ending IP address
   * applied to this firewall rule.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  FirewallRuleOperations.prototype.create = function(serverName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.endIPAddress === null || parameters.endIPAddress === undefined) {
      return callback(new Error('parameters.endIPAddress cannot be null.'));
    }
    if (parameters.name === null || parameters.name === undefined) {
      return callback(new Error('parameters.name cannot be null.'));
    }
    if (parameters.startIPAddress === null || parameters.startIPAddress === undefined) {
      return callback(new Error('parameters.startIPAddress cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/firewallrules';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var serviceResourceElement = js2xml.createElement('ServiceResource', 'http://schemas.microsoft.com/windowsazure');
    js2xml.addChildElement(requestDoc, serviceResourceElement);
    
    var nameElement = js2xml.createElement('Name', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(nameElement, parameters.name);
    js2xml.addChildElement(serviceResourceElement, nameElement);
    
    var startIPAddressElement = js2xml.createElement('StartIPAddress', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(startIPAddressElement, parameters.startIPAddress);
    js2xml.addChildElement(serviceResourceElement, startIPAddressElement);
    
    var endIPAddressElement = js2xml.createElement('EndIPAddress', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(endIPAddressElement, parameters.endIPAddress);
    js2xml.addChildElement(serviceResourceElement, endIPAddressElement);
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 201) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourceElement2 = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourceElement2) {
          var serviceResourceInstance = {};
          result.firewallRule = serviceResourceInstance;
          
          var nameElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement2) {
            var nameInstance = nameElement2;
            serviceResourceInstance.name = nameInstance;
          }
          
          var typeElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (typeElement) {
            var typeInstance = typeElement;
            serviceResourceInstance.type = typeInstance;
          }
          
          var startIPAddressElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'STARTIPADDRESS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (startIPAddressElement2) {
            var startIPAddressInstance = startIPAddressElement2;
            serviceResourceInstance.startIPAddress = startIPAddressInstance;
          }
          
          var endIPAddressElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'ENDIPADDRESS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (endIPAddressElement2) {
            var endIPAddressInstance = endIPAddressElement2;
            serviceResourceInstance.endIPAddress = endIPAddressInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Deletes a server-level firewall rule from a SQL Database server that
   * belongs to a subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715277.aspx for
   * more information)
   * 
   * @param {string} serverName The name of the server that will be have new
   * firewall rule applied to it.
   * 
   * @param {string} ruleName The name of the new firewall rule.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  FirewallRuleOperations.prototype.delete = function(serverName, ruleName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (ruleName === null || ruleName === undefined) {
      return callback(new Error('ruleName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/firewallrules/' + ruleName.trim();
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'DELETE';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Returns a list of all the server-level firewall rules for a SQL Database
   * server that belongs to a subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715278.aspx for
   * more information)
   * 
   * @param {string} serverName The name of the server for which the call is
   * being made.
   * 
   * @param {string} ruleName The name of the rule for which the call is being
   * made.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  FirewallRuleOperations.prototype.get = function(serverName, ruleName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (ruleName === null || ruleName === undefined) {
      return callback(new Error('ruleName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/firewallrules/' + ruleName.trim();
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourceElement = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourceElement) {
          var serviceResourceInstance = {};
          result.firewallRule = serviceResourceInstance;
          
          var nameElement = js2xml.getElement(responseDoc, serviceResourceElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement) {
            var nameInstance = nameElement;
            serviceResourceInstance.name = nameInstance;
          }
          
          var typeElement = js2xml.getElement(responseDoc, serviceResourceElement, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (typeElement) {
            var typeInstance = typeElement;
            serviceResourceInstance.type = typeInstance;
          }
          
          var startIPAddressElement = js2xml.getElement(responseDoc, serviceResourceElement, 'STARTIPADDRESS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (startIPAddressElement) {
            var startIPAddressInstance = startIPAddressElement;
            serviceResourceInstance.startIPAddress = startIPAddressInstance;
          }
          
          var endIPAddressElement = js2xml.getElement(responseDoc, serviceResourceElement, 'ENDIPADDRESS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (endIPAddressElement) {
            var endIPAddressInstance = endIPAddressElement;
            serviceResourceInstance.endIPAddress = endIPAddressInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Returns a list of all the server-level firewall rules for a SQL Database
   * server that belongs to a subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715278.aspx for
   * more information)
   * 
   * @param {string} serverName The name of the server for which the call is
   * being made.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  FirewallRuleOperations.prototype.list = function(serverName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/firewallrules';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { firewallRules: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourcesSequenceElement = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourcesSequenceElement) {
          for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, serviceResourcesSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
            var serviceResourcesElement = js2xml.getElements(responseDoc, serviceResourcesSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
            var serviceResourceInstance = {};
            result.firewallRules.push(serviceResourceInstance);
            
            var nameElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (nameElement) {
              var nameInstance = nameElement;
              serviceResourceInstance.name = nameInstance;
            }
            
            var typeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (typeElement) {
              var typeInstance = typeElement;
              serviceResourceInstance.type = typeInstance;
            }
            
            var startIPAddressElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'STARTIPADDRESS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (startIPAddressElement) {
              var startIPAddressInstance = startIPAddressElement;
              serviceResourceInstance.startIPAddress = startIPAddressInstance;
            }
            
            var endIPAddressElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ENDIPADDRESS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (endIPAddressElement) {
              var endIPAddressInstance = endIPAddressElement;
              serviceResourceInstance.endIPAddress = endIPAddressInstance;
            }
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Updates an existing server-level firewall rule for a SQL Database server
   * that belongs to a subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715280.aspx for
   * more information)
   * 
   * @param {string} serverName The name of the SQL database server to which
   * this rule will be applied.
   * 
   * @param {string} ruleName The name of the firewall rule to be updated.
   * 
   * @param {FirewallRuleUpdateParameters} parameters Parameters for the Update
   * Firewall Rule operation.
   * 
   * @param {string} parameters.name Gets or sets the name of this firewall
   * rule.
   * 
   * @param {string} parameters.startIPAddress Gets or sets the beginning IP
   * address applied to this firewall rule.
   * 
   * @param {string} parameters.endIPAddress Gets or sets the ending IP address
   * applied to this firewall rule.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  FirewallRuleOperations.prototype.update = function(serverName, ruleName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (ruleName === null || ruleName === undefined) {
      return callback(new Error('ruleName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.endIPAddress === null || parameters.endIPAddress === undefined) {
      return callback(new Error('parameters.endIPAddress cannot be null.'));
    }
    if (parameters.name === null || parameters.name === undefined) {
      return callback(new Error('parameters.name cannot be null.'));
    }
    if (parameters.startIPAddress === null || parameters.startIPAddress === undefined) {
      return callback(new Error('parameters.startIPAddress cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/firewallrules/' + ruleName.trim();
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'PUT';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var serviceResourceElement = js2xml.createElement('ServiceResource', 'http://schemas.microsoft.com/windowsazure');
    js2xml.addChildElement(requestDoc, serviceResourceElement);
    
    var nameElement = js2xml.createElement('Name', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(nameElement, parameters.name);
    js2xml.addChildElement(serviceResourceElement, nameElement);
    
    var startIPAddressElement = js2xml.createElement('StartIPAddress', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(startIPAddressElement, parameters.startIPAddress);
    js2xml.addChildElement(serviceResourceElement, startIPAddressElement);
    
    var endIPAddressElement = js2xml.createElement('EndIPAddress', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(endIPAddressElement, parameters.endIPAddress);
    js2xml.addChildElement(serviceResourceElement, endIPAddressElement);
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourceElement2 = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourceElement2) {
          var serviceResourceInstance = {};
          result.firewallRule = serviceResourceInstance;
          
          var nameElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement2) {
            var nameInstance = nameElement2;
            serviceResourceInstance.name = nameInstance;
          }
          
          var typeElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (typeElement) {
            var typeInstance = typeElement;
            serviceResourceInstance.type = typeInstance;
          }
          
          var startIPAddressElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'STARTIPADDRESS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (startIPAddressElement2) {
            var startIPAddressInstance = startIPAddressElement2;
            serviceResourceInstance.startIPAddress = startIPAddressInstance;
          }
          
          var endIPAddressElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'ENDIPADDRESS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (endIPAddressElement2) {
            var endIPAddressInstance = endIPAddressElement2;
            serviceResourceInstance.endIPAddress = endIPAddressInstance;
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  return FirewallRuleOperations;
})();

var ServerOperations = ( /** @lends ServerOperations */ function() {
  /**
   * @class
   * The SQL Database Management API includes operations for managing SQL
   * Database servers for a subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715271.aspx for
   * more information)
   * __NOTE__: An instance of this class is automatically created for an
   * instance of the [SqlManagementClient] {@link
   * SqlManagementClient~SqlManagementClient}.
   * See [servers] {@link SqlManagementClient~SqlManagementClient#servers}.
   * Initializes a new instance of the ServerOperations class.
   * @constructor
   * 
   * @param {SqlManagementClient} client Reference to the service client.
   */
  function ServerOperations(client) {
    this.client = client;
  }
  
  /**
   * Sets the administrative password of a SQL Database server for a
   * subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715272.aspx for
   * more information)
   * 
   * @param {string} serverName The server that will have the change made to
   * the administrative user.
   * 
   * @param {ServerChangeAdministratorPasswordParameters} parameters Parameters
   * for the Manage Administrator Password operation.
   * 
   * @param {string} parameters.newPassword Gets or sets new password for the
   * administrator.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  ServerOperations.prototype.changeAdministratorPassword = function(serverName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.newPassword === null || parameters.newPassword === undefined) {
      return callback(new Error('parameters.newPassword cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '?op=ResetPassword';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var administratorLoginPasswordElement = js2xml.createElement('AdministratorLoginPassword', 'http://schemas.microsoft.com/sqlazure/2010/12/');
    js2xml.addChildElement(requestDoc, administratorLoginPasswordElement);
    
    js2xml.setElementValue(administratorLoginPasswordElement, parameters.newPassword);
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Adds a new SQL Database server to a subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715274.aspx for
   * more information)
   * 
   * @param {ServerCreateParameters} parameters Parameters supplied to the
   * Create Server operation.
   * 
   * @param {string} parameters.administratorUserName Gets or sets the
   * administrator username.
   * 
   * @param {string} parameters.administratorPassword Gets or sets the
   * administrator password.
   * 
   * @param {string} parameters.location Gets or sets the region in which this
   * server will be created.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  ServerOperations.prototype.create = function(parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.administratorPassword === null || parameters.administratorPassword === undefined) {
      return callback(new Error('parameters.administratorPassword cannot be null.'));
    }
    if (parameters.administratorUserName === null || parameters.administratorUserName === undefined) {
      return callback(new Error('parameters.administratorUserName cannot be null.'));
    }
    if (parameters.location === null || parameters.location === undefined) {
      return callback(new Error('parameters.location cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var serverElement = js2xml.createElement('Server', 'http://schemas.microsoft.com/sqlazure/2010/12/');
    js2xml.addChildElement(requestDoc, serverElement);
    
    var administratorLoginElement = js2xml.createElement('AdministratorLogin', 'http://schemas.microsoft.com/sqlazure/2010/12/');
    js2xml.setElementValue(administratorLoginElement, parameters.administratorUserName);
    js2xml.addChildElement(serverElement, administratorLoginElement);
    
    var administratorLoginPasswordElement = js2xml.createElement('AdministratorLoginPassword', 'http://schemas.microsoft.com/sqlazure/2010/12/');
    js2xml.setElementValue(administratorLoginPasswordElement, parameters.administratorPassword);
    js2xml.addChildElement(serverElement, administratorLoginPasswordElement);
    
    var locationElement = js2xml.createElement('Location', 'http://schemas.microsoft.com/sqlazure/2010/12/');
    js2xml.setElementValue(locationElement, parameters.location);
    js2xml.addChildElement(serverElement, locationElement);
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.isBuffer(requestContent) ? requestContent.length : Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 201) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serverNameElement = js2xml.getElement(responseDoc, responseDoc, 'SERVERNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/');
        if (serverNameElement) {
          result.serverName = serverNameElement;
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Drops a SQL Database server from a subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715285.aspx for
   * more information)
   * 
   * @param {string} serverName The name of the server to be deleted.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  ServerOperations.prototype.delete = function(serverName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim();
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'DELETE';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Returns all SQL Database servers that are provisioned for a subscription.
   * (see http://msdn.microsoft.com/en-us/library/windowsazure/gg715269.aspx
   * for more information)
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  ServerOperations.prototype.list = function(callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = '/' + this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { servers: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serversSequenceElement = js2xml.getElement(responseDoc, responseDoc, 'SERVERS', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/');
        if (serversSequenceElement) {
          for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, serversSequenceElement, 'SERVER', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/').length; loweredIndex1 = loweredIndex1 + 1) {
            var serversElement = js2xml.getElements(responseDoc, serversSequenceElement, 'SERVER', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/')[loweredIndex1];
            var serverInstance = { features: {} };
            result.servers.push(serverInstance);
            
            var nameElement = js2xml.getElement(responseDoc, serversElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/');
            if (nameElement) {
              var nameInstance = nameElement;
              serverInstance.name = nameInstance;
            }
            
            var administratorLoginElement = js2xml.getElement(responseDoc, serversElement, 'ADMINISTRATORLOGIN', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/');
            if (administratorLoginElement) {
              var administratorLoginInstance = administratorLoginElement;
              serverInstance.administratorUserName = administratorLoginInstance;
            }
            
            var locationElement = js2xml.getElement(responseDoc, serversElement, 'LOCATION', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/');
            if (locationElement) {
              var locationInstance = locationElement;
              serverInstance.location = locationInstance;
            }
            
            var featuresSequenceElement = js2xml.getElement(responseDoc, serversElement, 'FEATURES', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/');
            if (featuresSequenceElement) {
              for (var loweredIndex2 = 0; loweredIndex2 < js2xml.getElements(responseDoc, featuresSequenceElement, 'FEATURE', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/').length; loweredIndex2 = loweredIndex2 + 1) {
                var featuresElement = js2xml.getElements(responseDoc, featuresSequenceElement, 'FEATURE', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/')[loweredIndex2];
                var featuresKey = js2xml.getElement(responseDoc, featuresElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/');
                var featuresValue = js2xml.getElement(responseDoc, featuresElement, 'VALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/');
                serverInstance.features[featuresKey] = featuresValue;
              }
            }
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  return ServerOperations;
})();

var ServiceObjectiveOperations = ( /** @lends ServiceObjectiveOperations */ function() {
  /**
   * @class
   * The SQL Database Management API includes operations for getting Service
   * Objective for a subscription.
   * __NOTE__: An instance of this class is automatically created for an
   * instance of the [SqlManagementClient] {@link
   * SqlManagementClient~SqlManagementClient}.
   * See [serviceObjectives] {@link
   * SqlManagementClient~SqlManagementClient#serviceObjectives}.
   * Initializes a new instance of the ServiceObjectiveOperations class.
   * @constructor
   * 
   * @param {SqlManagementClient} client Reference to the service client.
   */
  function ServiceObjectiveOperations(client) {
    this.client = client;
  }
  
  /**
   * Returns information about a certain Service Objective on a given Id.
   * 
   * @param {string} serverName The name of the SQL Server to be queried.
   * 
   * @param {string} serviceObjectiveId The Id of the Service Objective to be
   * obtained.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  ServiceObjectiveOperations.prototype.get = function(serverName, serviceObjectiveId, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (serviceObjectiveId === null || serviceObjectiveId === undefined) {
      return callback(new Error('serviceObjectiveId cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/serviceobjectives/' + serviceObjectiveId.trim();
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = {};
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourceElement = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourceElement) {
          var serviceResourceInstance = { dimensionSettings: [] };
          result.serviceObjective = serviceResourceInstance;
          
          var nameElement = js2xml.getElement(responseDoc, serviceResourceElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement) {
            var nameInstance = nameElement;
            serviceResourceInstance.name = nameInstance;
          }
          
          var typeElement = js2xml.getElement(responseDoc, serviceResourceElement, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (typeElement) {
            var typeInstance = typeElement;
            serviceResourceInstance.type = typeInstance;
          }
          
          var stateElement = js2xml.getElement(responseDoc, serviceResourceElement, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (stateElement) {
            var stateInstance = stateElement;
            serviceResourceInstance.state = stateInstance;
          }
          
          var selfLinkElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SELFLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (selfLinkElement) {
            var selfLinkInstance = selfLinkElement;
            serviceResourceInstance.selfLink = selfLinkInstance;
          }
          
          var parentLinkElement = js2xml.getElement(responseDoc, serviceResourceElement, 'PARENTLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (parentLinkElement) {
            var parentLinkInstance = parentLinkElement;
            serviceResourceInstance.parentLink = parentLinkInstance;
          }
          
          var idElement = js2xml.getElement(responseDoc, serviceResourceElement, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (idElement) {
            var idInstance = idElement;
            serviceResourceInstance.id = idInstance;
          }
          
          var isDefaultElement = js2xml.getElement(responseDoc, serviceResourceElement, 'ISDEFAULT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (isDefaultElement) {
            var isDefaultInstance = isDefaultElement === 'true';
            serviceResourceInstance.isDefault = isDefaultInstance;
          }
          
          var isSystemElement = js2xml.getElement(responseDoc, serviceResourceElement, 'ISSYSTEM', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (isSystemElement) {
            var isSystemInstance = isSystemElement === 'true';
            serviceResourceInstance.isSystem = isSystemInstance;
          }
          
          var descriptionElement = js2xml.getElement(responseDoc, serviceResourceElement, 'DESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (descriptionElement) {
            var descriptionInstance = descriptionElement;
            serviceResourceInstance.description = descriptionInstance;
          }
          
          var enabledElement = js2xml.getElement(responseDoc, serviceResourceElement, 'ENABLED', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (enabledElement) {
            var enabledInstance = enabledElement === 'true';
            serviceResourceInstance.enabled = enabledInstance;
          }
          
          var dimensionSettingsSequenceElement = js2xml.getElement(responseDoc, serviceResourceElement, 'DIMENSIONSETTINGS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (dimensionSettingsSequenceElement) {
            for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, dimensionSettingsSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
              var dimensionSettingsElement = js2xml.getElements(responseDoc, dimensionSettingsSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
              var serviceResourceInstance2 = {};
              serviceResourceInstance.dimensionSettings.push(serviceResourceInstance2);
              
              var nameElement2 = js2xml.getElement(responseDoc, dimensionSettingsElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (nameElement2) {
                var nameInstance2 = nameElement2;
                serviceResourceInstance2.name = nameInstance2;
              }
              
              var typeElement2 = js2xml.getElement(responseDoc, dimensionSettingsElement, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (typeElement2) {
                var typeInstance2 = typeElement2;
                serviceResourceInstance2.type = typeInstance2;
              }
              
              var stateElement2 = js2xml.getElement(responseDoc, dimensionSettingsElement, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (stateElement2) {
                var stateInstance2 = stateElement2;
                serviceResourceInstance2.state = stateInstance2;
              }
              
              var selfLinkElement2 = js2xml.getElement(responseDoc, dimensionSettingsElement, 'SELFLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (selfLinkElement2) {
                var selfLinkInstance2 = selfLinkElement2;
                serviceResourceInstance2.selfLink = selfLinkInstance2;
              }
              
              var parentLinkElement2 = js2xml.getElement(responseDoc, dimensionSettingsElement, 'PARENTLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (parentLinkElement2) {
                var parentLinkInstance2 = parentLinkElement2;
                serviceResourceInstance2.parentLink = parentLinkInstance2;
              }
              
              var idElement2 = js2xml.getElement(responseDoc, dimensionSettingsElement, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (idElement2) {
                var idInstance2 = idElement2;
                serviceResourceInstance2.id = idInstance2;
              }
              
              var descriptionElement2 = js2xml.getElement(responseDoc, dimensionSettingsElement, 'DESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (descriptionElement2) {
                var descriptionInstance2 = descriptionElement2;
                serviceResourceInstance2.description = descriptionInstance2;
              }
              
              var ordinalElement = js2xml.getElement(responseDoc, dimensionSettingsElement, 'ORDINAL', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (ordinalElement) {
                var ordinalInstance = parseInt(ordinalElement, 10);
                serviceResourceInstance2.ordinal = ordinalInstance;
              }
              
              var isDefaultElement2 = js2xml.getElement(responseDoc, dimensionSettingsElement, 'ISDEFAULT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (isDefaultElement2) {
                var isDefaultInstance2 = isDefaultElement2 === 'true';
                serviceResourceInstance2.isDefault = isDefaultInstance2;
              }
            }
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * Returns information about all Service Objectives on a database server.
   * 
   * @param {string} serverName The name of the database server to be queried.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  ServiceObjectiveOperations.prototype.list = function(serverName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var baseUrl = this.client.baseUri;
    var url2 = this.client.credentials.subscriptionId.trim() + '/services/sqlservers/servers/' + serverName.trim() + '/serviceobjectives';
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { serviceObjectives: [] };
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourcesSequenceElement = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourcesSequenceElement) {
          for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, serviceResourcesSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
            var serviceResourcesElement = js2xml.getElements(responseDoc, serviceResourcesSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
            var serviceResourceInstance = { dimensionSettings: [] };
            result.serviceObjectives.push(serviceResourceInstance);
            
            var nameElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (nameElement) {
              var nameInstance = nameElement;
              serviceResourceInstance.name = nameInstance;
            }
            
            var typeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (typeElement) {
              var typeInstance = typeElement;
              serviceResourceInstance.type = typeInstance;
            }
            
            var stateElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (stateElement) {
              var stateInstance = stateElement;
              serviceResourceInstance.state = stateInstance;
            }
            
            var selfLinkElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SELFLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (selfLinkElement) {
              var selfLinkInstance = selfLinkElement;
              serviceResourceInstance.selfLink = selfLinkInstance;
            }
            
            var parentLinkElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'PARENTLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (parentLinkElement) {
              var parentLinkInstance = parentLinkElement;
              serviceResourceInstance.parentLink = parentLinkInstance;
            }
            
            var idElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (idElement) {
              var idInstance = idElement;
              serviceResourceInstance.id = idInstance;
            }
            
            var isDefaultElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ISDEFAULT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (isDefaultElement) {
              var isDefaultInstance = isDefaultElement === 'true';
              serviceResourceInstance.isDefault = isDefaultInstance;
            }
            
            var isSystemElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ISSYSTEM', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (isSystemElement) {
              var isSystemInstance = isSystemElement === 'true';
              serviceResourceInstance.isSystem = isSystemInstance;
            }
            
            var descriptionElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'DESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (descriptionElement) {
              var descriptionInstance = descriptionElement;
              serviceResourceInstance.description = descriptionInstance;
            }
            
            var enabledElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ENABLED', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (enabledElement) {
              var enabledInstance = enabledElement === 'true';
              serviceResourceInstance.enabled = enabledInstance;
            }
            
            var dimensionSettingsSequenceElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'DIMENSIONSETTINGS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (dimensionSettingsSequenceElement) {
              for (var loweredIndex2 = 0; loweredIndex2 < js2xml.getElements(responseDoc, dimensionSettingsSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex2 = loweredIndex2 + 1) {
                var dimensionSettingsElement = js2xml.getElements(responseDoc, dimensionSettingsSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex2];
                var serviceResourceInstance2 = {};
                serviceResourceInstance.dimensionSettings.push(serviceResourceInstance2);
                
                var nameElement2 = js2xml.getElement(responseDoc, dimensionSettingsElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (nameElement2) {
                  var nameInstance2 = nameElement2;
                  serviceResourceInstance2.name = nameInstance2;
                }
                
                var typeElement2 = js2xml.getElement(responseDoc, dimensionSettingsElement, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (typeElement2) {
                  var typeInstance2 = typeElement2;
                  serviceResourceInstance2.type = typeInstance2;
                }
                
                var stateElement2 = js2xml.getElement(responseDoc, dimensionSettingsElement, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (stateElement2) {
                  var stateInstance2 = stateElement2;
                  serviceResourceInstance2.state = stateInstance2;
                }
                
                var selfLinkElement2 = js2xml.getElement(responseDoc, dimensionSettingsElement, 'SELFLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (selfLinkElement2) {
                  var selfLinkInstance2 = selfLinkElement2;
                  serviceResourceInstance2.selfLink = selfLinkInstance2;
                }
                
                var parentLinkElement2 = js2xml.getElement(responseDoc, dimensionSettingsElement, 'PARENTLINK', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (parentLinkElement2) {
                  var parentLinkInstance2 = parentLinkElement2;
                  serviceResourceInstance2.parentLink = parentLinkInstance2;
                }
                
                var idElement2 = js2xml.getElement(responseDoc, dimensionSettingsElement, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (idElement2) {
                  var idInstance2 = idElement2;
                  serviceResourceInstance2.id = idInstance2;
                }
                
                var descriptionElement2 = js2xml.getElement(responseDoc, dimensionSettingsElement, 'DESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (descriptionElement2) {
                  var descriptionInstance2 = descriptionElement2;
                  serviceResourceInstance2.description = descriptionInstance2;
                }
                
                var ordinalElement = js2xml.getElement(responseDoc, dimensionSettingsElement, 'ORDINAL', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (ordinalElement) {
                  var ordinalInstance = parseInt(ordinalElement, 10);
                  serviceResourceInstance2.ordinal = ordinalInstance;
                }
                
                var isDefaultElement2 = js2xml.getElement(responseDoc, dimensionSettingsElement, 'ISDEFAULT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
                if (isDefaultElement2) {
                  var isDefaultInstance2 = isDefaultElement2 === 'true';
                  serviceResourceInstance2.isDefault = isDefaultInstance2;
                }
              }
            }
          }
        }
        
        result.statusCode = statusCode;
        result.requestId = response.headers['x-ms-request-id'];
        
        return callback(null, result);
      });
    });
  };
  
  return ServiceObjectiveOperations;
})();
